/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2016 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXSSHFS, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef CallBacks_H
#define CallBacks_H

#undef WINVER
#define WINVER 0x500
#define _WIN32_IE 0x0500

#include <cstdio>
#include <windows.h>
#include <wchar.h>
#include <set>
#include <Shellapi.h>
#include <shlobj.h>
#include <Shlwapi.h>

#include "DokanClient.h"
#include "Types.h"
#include "Debug.h"
#include "SftpClient.h"
#include "Cache.h"
#include "Options.h"
#include "dokan.h"

//
// Typedef.
//

typedef IShellFolder2 *LPSHELLFOLDER2;

//
// Functions.
//

void GetFullRemotePath(Char *fullPath, wchar_t *path, 
                           Int size, Int overwrite = 0);

void GetFullRemotePath(Char *fullPath, const wchar_t *path, Int size);

//
// I/O Callbacks. We redirect I/O operations to Sftp here.
//

int DOKAN_CALLBACK NxCreateFile(LPCWSTR FileName, DWORD AccessMode, DWORD ShareMode,
                                        DWORD CreationDisposition, DWORD FlagsAndAttributes,
                                            PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxCreateDirectory(LPCWSTR FileName, PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxOpenDirectory(LPCWSTR FileName, PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxCloseFile(LPCWSTR FileName, PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxCleanup(LPCWSTR FileName, PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxReadFile(LPCWSTR FileName, LPVOID Buffer, DWORD BufferLength,
                                      LPDWORD ReadLength, LONGLONG Offset,
                                          PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxWriteFile(LPCWSTR FileName, LPCVOID Buffer, DWORD NumberOfBytesToWrite,
                                       LPDWORD NumberOfBytesWritten, LONGLONG Offset,
                                           PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxFlushFileBuffers(LPCWSTR FileName, PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxGetFileInformation(LPCWSTR FileName, 
                                            LPBY_HANDLE_FILE_INFORMATION HandleFileInformation,
                                                PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxFindFiles(LPCWSTR FileName, PFillFindData FillFindData,
                                   PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxDeleteFile(LPCWSTR FileName, PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxDeleteDirectory(LPCWSTR FileName, 
                                         PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxMoveFile(LPCWSTR FileName, LPCWSTR NewFileName,
                                  BOOL ReplaceIfExisting, 
                                      PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxLockFile(LPCWSTR FileName, LONGLONG ByteOffset,
                                  LONGLONG Length, PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxSetEndOfFile(LPCWSTR FileName, LONGLONG ByteOffset,
                                      PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxSetAllocationSize(LPCWSTR FileName, LONGLONG AllocSize,
                                          PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxSetFileAttributes(LPCWSTR FileName, DWORD FileAttributes,
                                           PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxSetFileTime(LPCWSTR FileName, CONST FILETIME *CreationTime,
                                     CONST FILETIME *LastAccessTime,
                                         CONST FILETIME *LastWriteTime,
                                             PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxUnlockFile(LPCWSTR FileName, LONGLONG ByteOffset,
                                    LONGLONG Length, PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxGetFileSecurity(LPCWSTR FileName, PSECURITY_INFORMATION si,
                                             PSECURITY_DESCRIPTOR sd,
                                                 ULONG bufSize, PULONG sizeNeeded,
                                                     PDOKAN_FILE_INFO dokanInfo);

int DOKAN_CALLBACK NxSetFileSecurity(LPCWSTR FileName, PSECURITY_INFORMATION si,
                                         PSECURITY_DESCRIPTOR sd, ULONG sdSize,
                                             PDOKAN_FILE_INFO dokanInfo);

int DOKAN_CALLBACK NxGetDiskFreeSpace(PULONGLONG freeBytesAvailable,
                                          PULONGLONG totalNumberOfBytes,
                                              PULONGLONG totalNumberOfFreeBytes,
                                                  PDOKAN_FILE_INFO dokanInfo);

int DOKAN_CALLBACK NxGetVolumeInformation(LPWSTR VolumeNameBuffer, DWORD VolumeNameSize,
                                                  LPDWORD VolumeSerialNumber,
                                                      LPDWORD MaximumComponentLength,
                                                          LPDWORD FileSystemFlags,
                                                              LPWSTR FileSystemNameBuffer,
                                                                  DWORD FileSystemNameSize,
                                                                      PDOKAN_FILE_INFO DokanFileInfo);

int DOKAN_CALLBACK NxUnmount(PDOKAN_FILE_INFO DokanFileInfo);

Int RecyclerFlush(const wchar_t *rootIn = NULL);

#endif /* Callbacks_H */
