/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2016 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXSSHFS, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef Debug_H
#define Debug_H

#undef STRING

#include "String.h"
#include "Types.h"
#include "Utils.h"

#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

//
// #define DEBUG flag to enable compilation of debug code.
//

#define DEBUG 
#define TRACE
#undef  PACKET_DUMP

#undef  SEPARATE_LOGS
#undef  RECONNECT_FAKE

#undef  EMULATE_SLOW_NETWORK
  
//
// Macros for errors catching.
//

#define FAIL(CONDITION) if(CONDITION) goto fail

#define HFAIL(X) if ((hr = (X)) != S_OK) goto fail

#define FAILEX(CONDITION, ...) if(CONDITION) {Error(__VA_ARGS__); goto fail;}

#define NTFAIL(NTFUNC) if((ntStat = (NTFUNC))) goto fail

#define DOKAN_FAIL(CODE) {dokanCode = -CODE; goto fail;}

#define FAIL_MSG(VALUE) {StringSet(errorMessage, VALUE); goto fail;}

//
// Macros and functions for debug messages.
//

#ifdef DEBUG

  #include <windows.h>
  #include <cstdarg>
  #include <cstdio>
 
  //
  // Trace enter/leave to/from functions in log.
  //
  
  #ifdef TRACE
  
    #define DBG_ENTER(FUNC_NAME) DbgEnter(FUNC_NAME)
                          
    #define DBG_LEAVE(FUNC_NAME) DbgLeave(FUNC_NAME)
  
    void DbgEnter(const Char *funcName);

    void DbgLeave(const Char *funcName);

  #else

    #define DBG_ENTER(FUNC_NAME)
                          
    #define DBG_LEAVE(FUNC_NAME)
     
  #endif
  
  //
  // Dump raw input/output packets to log.
  //
  
  #ifdef PACKET_DUMP
  
    #define DBG_BUFFER_DUMP(buf, size) DbgDumpBuffer(buf, size)
    
    void DbgDumpBuffer(const void *inpBuf, Int size);

  #else
  
    #define DBG_BUFFER_DUMP(buf, size)

  #endif  
    
  //
  // General log messages.
  //
  
  #define DBG_INIT(PATH) DbgInit(PATH)  

  #define DBG_STOP() DbgStop()
  
  #define DBG_MSG(FMT, ARGS...) DbgMsg(2, FMT, ## ARGS)

  #define DBG_SET_TREE_MODE(state) DbgTreeMode = (state)
  
  #define DBG_DELETE_LOG() DbgDeleteLog()
  
  #define DBG_SET_LEVEL(X) DbgSetLevel(X)
  
  #define DBG_GET_LEVEL() DbgGetLevel()


  void DbgInit(const Char *path);

  void DbgStop();

  void DbgMsg(int level, const Char *fmt, ...);

  void DbgDeleteLog();
  
  void DbgSetLevel(Int level);
  
  Int DbgGetLevel();
  
#else

  //
  // When no debug, we define only "ghost function" macros.
  //
  
  #define DBG_INIT(PATH) 
  
  #define DBG_ENTER(FUNC_NAME) 

  #define DBG_LEAVE(FUNC_NAME) 
  
  #define DBG_MSG(FMT, ARGS...) 

  #define DBG_SET_TREE_MODE(state) 
  
  #define DBG_BUFFER_DUMP(buf, size)
  
  #define DBG_DELETE_LOG()
  
  #define DBG_SET_LEVEL(X)

  #define DBG_GET_LEVEL() 0
  
#endif

void DbgError(const Char *fmt, ...);

//
// Write error message on stderr and optionally to log file.
//

#define Error(...) {DbgError(__VA_ARGS__); DbgMsg(1, __VA_ARGS__);}

//
// Write error message and send SIGBREAK to itself.
//

#define Fatal(...) {Error(__VA_ARGS__); StopProcessSafely();}

#endif
