/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2016 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXSSHFS, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef IOFifo_H
#define IOFifo_H

//
// Includes.
//

#include <windows.h>
#include <cstdio>
#include <queue>

#include "Debug.h"
#include "Utils.h"
#include "Sftp.h"

//
// Defines.
//

#define IOFifoLock() {DBG_ENTER("IOFifoLock"); WaitForSingleObject(IOFifoMutex, INFINITE);}
#define IOFifoUnlock() {DBG_LEAVE("IOFifoLock"); ReleaseMutex(IOFifoMutex);}

#define IdCountLock() WaitForSingleObject(IdCountMutex, INFINITE)
#define IdCountUnlock() ReleaseMutex(IdCountMutex)

#define LOCK_OBJECT(X) WaitForSingleObject(X##Mutex, INFINITE)
#define UNLOCK_OBJECT(X) ReleaseMutex(X##Mutex)

#define MAX_PACKET (1024 * 128)

#define ATOMIC_TIMEOUT 10000

#define MAX_RECONNECTS 3

//
// States of I/O request.
//

//
// Request waits for it's time.
//

#define WAIT_STATE    0

//
// Poped from FIFO, worker is processing it now.
//

#define PENDING_STATE 1

//
// Request finished with success.
//

#define SUCCESS_STATE 2

//
// Request finished with error.
//

#define ERROR_STATE   3 

//
// Structs.
//

struct IOFifoEntry
{
  Int id_;
  
  Int outSize_;

  union
  {
    SftpMsg *outData_;
    SftpMsg **poutData_;
  };
  
  SftpMsg *inpData_;
  
  Int dynamic_;
  Int inpFree_;
  Int state_;
  
  HANDLE readyEvent_;
};

//
// Exported functions.
//

Int AtomicWrite(HANDLE handle, const void *outBuf, 
                    Int bytesToWrite, Int timeout = ATOMIC_TIMEOUT);

Int AtomicRead(HANDLE handle, void *inpBuf, 
                   Int bytesToRead, Int timeout = ATOMIC_TIMEOUT);

Int IOFifoThreadHandler(void *unused);

Int IORequest(IOFifoEntry *iorq, Int timeout);

Int IOFifoInit(HANDLE stdinHandle, HANDLE stdoutHandle, 
                   HANDLE stderrHandle, HANDLE reconnHandle);
                   
Int IOFifoReconnect();

void IOFifoCleanup();

Int IOFifoGenerateUniqueId();

#endif /* IOFifo_H */
