/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2016 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXSSHFS, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include "Options.h"

//
// Parse command line arguments and sets structures field.
//
// argc - number of args in argv (IN)
// argv - string table with arguments (IN)
//
// RETURNS: 0 if OK.
//

Int OptionsStruct::parseArguments(Int argc, Char **argv)
{
  Int argc_w = 0;

  wchar_t *argv_w[argc];
  error_ = 1;

  mode_ = MOUNT_MODE;

  //
  // Changed string conversion.
  //

  for (Int i = 0; i < argc; i++)
  {
    Int size = MultiByteToWideChar(CP_UTF8, 0, argv[i], -1, NULL, 0);

    argv_w[i] = new WCHAR[size];

    MultiByteToWideChar(CP_UTF8, 0, argv[i], -1, (LPWSTR) argv_w[i], size);
  }

  for (Int i = 1; i < argc; i++)
  {
    Int argsToEnd = argc - i - 1;

    Int argsToSkip = 0;

    if (StringCompare(argv[i], "-d") == 0)
    {
      DBG_SET_LEVEL(3);
    }
    else if (StringCompare(argv[i], "-P") == 0)
    {
      FAIL(argsToEnd < 1);

      DBG_INIT(argv[i + 1]);

      argsToSkip = 1;
    }
    else if (StringChar(argv[i], '@') || StringFind(argv[i], "127.0.0.1"))
    {
      FAIL(argsToEnd < 1);

      char *hostPtr       = strchr(argv[i], '@');
      char *remoteRootPtr = strchr(argv[i], ':');

      wchar_t *hostPtr_w       = wcschr(argv_w[i], L'@');
      wchar_t *remoteRootPtr_w = wcschr(argv_w[i], L':');

      FAIL(remoteRootPtr_w == NULL);

      if (hostPtr != NULL)
      {
        hostPtr_w[0] = 0;
        hostPtr[0]   = 0;

        hostPtr_w ++;
        hostPtr ++;
      }
      else
      {
        hostPtr   = (Char *) "127.0.0.1";
        hostPtr_w = (wchar_t *) L"127.0.0.1";
      }

      remoteRootPtr_w[0] = 0;
      remoteRootPtr[0] = 0;

      remoteRootPtr_w ++;
      remoteRootPtr ++;

      //
      // We need user and host in ascii for ssh-client.
      //

      user_ = argv[i];
      host_ = hostPtr;

      //
      // And local and remote mount paths in unicode for dokan.
      //

      localRootA_ = argv[i + 1];

      wcscpy(localRoot_, argv_w[i + 1]);

      wcscpy(remoteRoot_, remoteRootPtr_w);

      NormalizePath(localRoot_, sizeof(localRoot_), L'/');

      NormalizePath(remoteRoot_, sizeof(remoteRoot_), L'/');

/*
      FIXME. Old implementation. This conversion goes bad on
             some machines for unknown reason.

      FAIL(MultiByteToWideChar(CP_THREAD_ACP, 0, argv[i + 1], -1,
                                   localRoot_, sizeof(localRoot_)) == FALSE);

      FAIL(MultiByteToWideChar(CP_THREAD_ACP, 0, remoteRootPtr, -1,
                                   remoteRoot_, sizeof(remoteRoot_)) == FALSE);
*/

      argsToSkip = 1;
    }

    //
    // -p <port>. Port number for ssh connection.
    //

    else if (StringCompare(argv[i], "-p") == 0)
    {
      FAIL(argsToEnd < 1);

      port_ = atoi(argv[i + 1]);

      argsToSkip = 1;
    }

    //
    // Optionally passed std handles.
    // If not passed we use stdin, stdout, stderr
    // for current process.
    //

    else if (StringCompare(argv[i], "--stdin") == 0)
    {
      FAIL(argsToEnd < 1);

      stdinHandle_ = (HANDLE) atoi(argv[i + 1]);

      FAIL(argsToEnd < 1);
    }
    else if (StringCompare(argv[i], "--stderr") == 0)
    {
      FAIL(argsToEnd < 1);

      stderrHandle_ = (HANDLE) atoi(argv[i + 1]);

      FAIL(argsToEnd < 1);
    }
    else if (StringCompare(argv[i], "--stdout") == 0)
    {
      FAIL(argsToEnd < 1);

      stdoutHandle_ = (HANDLE) atoi(argv[i + 1]);

      FAIL(argsToEnd < 1);
    }
    else if (StringCompare(argv[i], "--reconnect") == 0)
    {
      FAIL(argsToEnd < 1);

      reconnHandle_ = (HANDLE) atoi(argv[i + 1]);

      FAIL(argsToEnd < 1);
    }

    //
    // --timeout <t>. Sets timeout for waiting for server's responde.
    //

    else if (StringCompare(argv[i], "--timeout") == 0)
    {
      FAIL(argsToEnd < 1);

      timeout_ = atoi(argv[i + 1]);

      argsToSkip = 1;
    }

    //
    // --parentpid <pid>. Synchronize live with parent process.
    //

    else if (StringCompare(argv[i], "--parentpid") == 0)
    {
      FAIL(argsToEnd < 1);

      parentPid_ = atoi(argv[i + 1]);

      argsToSkip = 1;
    }

    //
    // --unmount x:
    //

    else if (StringCompare(argv[i], "--unmount") == 0 ||
                 StringCompare(argv[i], "-f") == 0 ||
                     StringCompare(argv[i], "-uz") == 0)
    {
      mode_ = UNMOUNT_MODE;

      FAIL(argsToEnd < 1);

      wcscpy(localRoot_, argv_w[i + 1]);

      NormalizePath(localRoot_, sizeof(localRoot_), L'/');

      argsToSkip = 1;
    }

    //
    // -i <keyfile>
    //

    else if (StringCompare(argv[i], "-i") == 0)
    {
      FAIL(argsToEnd < 1);

      keyfile_ = argv[i + 1];

      argsToSkip = 1;
    }

    //
    // -o slave
    //

    else if (StringCompare(argv[i], "-o") == 0)
    {
      FAIL(argsToEnd < 1);

      if (StringCompare(argv[i + 1], "slave") == 0)
      {
        slaveMode_ = 1;
      }

      argsToSkip = 1;
    }

    //
    // All other options we interpreate as extra options for ssh client
    // and we pass it to child without changes.
    //

    else
    {
      if (sshArgs_ == NULL)
      {
        sshArgs_ = (char **) malloc(sizeof(char *) * 2);
      }
      else
      {
        sshArgs_ = (char **) realloc(sshArgs_, sizeof(char *) * (sshArgsCount_ + 2));
      }

      sshArgs_[sshArgsCount_] = argv[i];

      sshArgsCount_ ++;
    }

    //
    // Next argument.
    //

    i += argsToSkip;
  }

  //
  // MOUNT_MODE only.
  //

  if (mode_ == MOUNT_MODE && Not(slaveMode_))
  {
    DWORD mode;

    //
    // Verify syntax.
    //

    FAIL(user_ == NULL);
    FAIL(host_ == NULL);
    FAIL(remoteRoot_ == NULL);
    FAIL(localRoot_ == NULL);

    //
    // FIXME. Handle key passphrase here.
    //

    if (keyfile_ == NULL)
    {
      //
      // Disable echo.
      //

      GetConsoleMode(GetStdHandle(STD_INPUT_HANDLE), &mode);

      SetConsoleMode(GetStdHandle(STD_INPUT_HANDLE), mode & (~ENABLE_ECHO_INPUT));

      //
      // Read password from stdin.
      //

      DBG_MSG("Reading password...\n");

      printf("Password: ");

      fflush(stdout);

      fgets(password_, sizeof(password_), stdin);

      //
      // Stupid encryption to avoid storing raw password in memory.
      //

      for (int i = 0; password_[i]; i++)
      {
        password_[i] ++;
      }

      printf("\n");

      fflush(stdout);

      //
      // Enable echo.
      //

      SetConsoleMode(GetStdHandle(STD_INPUT_HANDLE), mode | ENABLE_ECHO_INPUT);
    }
  }

  error_ = 0;

fail:

  if (error_)
  {
    Error("ERROR: Syntax error.\n");

    mode_ = ERROR_MODE;
  }

  for (Int i = 0; i < argc; i++)
  {
    delete [] argv_w[i];
  }

  return error_;
}


OptionsStruct::OptionsStruct()

: mode_(MOUNT_MODE),

  error_(0),

  slaveMode_(0),

  user_(NULL),

  host_(NULL),

  keyfile_(NULL),

  port_(22),

  parentPid_(-1),

  sshArgsCount_(0),

  sshArgs_(NULL),

  timeout_(10000)
{
  remoteRoot_[0] = 0;
  localRoot_[0]  = 0;
  password_[0]   = 0;

  localRootA_ = NULL;

  memset(remoteRoot_, 0, MAX_PATH * sizeof(wchar_t));
  memset(localRoot_,  0, MAX_PATH * sizeof(wchar_t));

  stdinHandle_  = GetStdHandle(STD_INPUT_HANDLE);
  stdoutHandle_ = GetStdHandle(STD_OUTPUT_HANDLE);
  stderrHandle_ = GetStdHandle(STD_ERROR_HANDLE);
  reconnHandle_ = NULL;
}

OptionsStruct::~OptionsStruct()
{
  if (sshArgs_ != NULL)
  {
    free(sshArgs_);
  }
}
