/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2016 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXSSHFS, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef Sftp_H
#define Sftp_H

#undef ERROR
#undef STRING

#include "Types.h"

//
// Our Sftp version.
//

#define SSH2_FILEXFER_VERSION 3

//
// Client to server messages.
//

#define SSH2_FXP_INIT           1
#define SSH2_FXP_OPEN           3
#define SSH2_FXP_CLOSE          4
#define SSH2_FXP_READ           5
#define SSH2_FXP_WRITE          6
#define SSH2_FXP_LSTAT          7
#define SSH2_FXP_STAT_VERSION_0 7
#define SSH2_FXP_FSTAT          8
#define SSH2_FXP_SETSTAT        9
#define SSH2_FXP_FSETSTAT       10
#define SSH2_FXP_OPENDIR        11
#define SSH2_FXP_READDIR        12
#define SSH2_FXP_REMOVE         13
#define SSH2_FXP_MKDIR          14
#define SSH2_FXP_RMDIR          15
#define SSH2_FXP_REALPATH       16
#define SSH2_FXP_STAT           17
#define SSH2_FXP_RENAME         18
#define SSH2_FXP_READLINK       19
#define SSH2_FXP_SYMLINK        20
#define SSH2_FXP_EXTENDED       200

//
// Server to client messages.
//

#define SSH2_FXP_VERSION 2
#define SSH2_FXP_STATUS  101
#define SSH2_FXP_HANDLE  102
#define SSH2_FXP_DATA    103
#define SSH2_FXP_NAME    104
#define SSH2_FXP_ATTRS   105

#define SSH2_FXP_EXTENDED_REPLY 201

//
// Attributes.
//

#define SSH2_FILEXFER_ATTR_SIZE        0x00000001
#define SSH2_FILEXFER_ATTR_UIDGID      0x00000002
#define SSH2_FILEXFER_ATTR_PERMISSIONS 0x00000004
#define SSH2_FILEXFER_ATTR_ACMODTIME   0x00000008
#define SSH2_FILEXFER_ATTR_EXTENDED    0x80000000

//
// Portable open modes.
//

#define SSH2_FXF_READ   0x00000001
#define SSH2_FXF_WRITE  0x00000002
#define SSH2_FXF_APPEND 0x00000004
#define SSH2_FXF_CREAT  0x00000008
#define SSH2_FXF_TRUNC  0x00000010
#define SSH2_FXF_EXCL   0x00000020

//
// statvfs@openssh.com f_flag flags.
//

#define SSH2_FXE_STATVFS_ST_RDONLY 0x00000001
#define SSH2_FXE_STATVFS_ST_NOSUID 0x00000002

//
// Status codes.
//

#define SSH2_FX_OK                0
#define SSH2_FX_EOF               1
#define SSH2_FX_NO_SUCH_FILE      2
#define SSH2_FX_PERMISSION_DENIED 3
#define SSH2_FX_FAILURE           4
#define SSH2_FX_BAD_MESSAGE       5
#define SSH2_FX_NO_CONNECTION     6
#define SSH2_FX_CONNECTION_LOST   7
#define SSH2_FX_OP_UNSUPPORTED    8
#define SSH2_FX_MAX               8

//
// Description of one sftp message. The same struct is
// used for sended and received messages.
//

struct SftpMsg
{
  Int size_;
  
  unsigned Char type_;
  
  Char data_[1];
}
__attribute__((packed));


struct SftpStrMsg
{
  Int size_;
  
  Char type_;

  Int id_;
  Int strLen_;

  Char str_[1];
}
__attribute__((packed));


struct SftpMkdirMsg 
{
  Int size_;
  
  Char type_;

  Int id_;
  Int pathLen_;

  Char path_[MAX_PATH];

  Int attribs_;
}
__attribute__((packed));


struct SftpNameMsg 
{
  Int size_;
  
  Char type_;

  Int id_;
  Int count_;

  Char data_[1];
}
__attribute__((packed));


struct SftpInitMsg
{
  Int size_;
  
  Char type_;

  Int version_;
}
__attribute__((packed));


struct SftpVersionMsg
{
  Int size_;
  
  Char type_;

  Int version_;

  //
  // FIXME. Here are more info about server, but
  // we don't need it in sshfs client.
  //
  
  Char data[1]; 
}
__attribute__((packed));


struct SftpStatusMsg 
{
  Int size_;
  
  Char type_;

  Int id_;
  Int status_;
  
  Char data_[128];
}
__attribute__((packed));


struct SftpAttrsMsg 
{
  Int size_;
  
  Char type_;

  Int id_;
  Int data_[1];
}
__attribute__((packed));


struct SftpHandleMsg
{
  Int size_;
  
  Char type_;

  Int id_;
  Int handleLen_;
  Int handle_[2];
}
__attribute__((packed));


struct SftpReadMsg
{
  Int size_;
  
  Char type_;

  Int id_;
  Int handleLen_;
  Int handle_;
  Int offsetHigh_;
  Int offsetLow_;  
  Int bytesToRead_;
}
__attribute__((packed));


struct SftpWriteMsg
{
  Int size_;
  
  Char type_;

  Int id_;
  Int handleLen_;
  Int handle_;
  Int offsetHigh_;
  Int offsetLow_;  
  Int bytesToWrite_;
  
  Char data_[1];
}
__attribute__((packed));


struct SftpOpenMsg
{
  Int size_;
  
  Char type_;

  Int id_;
  Int pathLen_;
  
  Char path_[MAX_PATH];

  Int mode_;
  Int attribs_;
}
__attribute__((packed));


struct SftpRenameMsg
{
  Int size_;
  
  Char type_;

  Int id_;
  Int path1Len_;
  
  Char path1_[MAX_PATH];

  Int path2Len_;
  
  Char path2_[MAX_PATH];
}
__attribute__((packed));

//
// "statvfs@openssh.com" extension.
//

struct SftpStatvfsMsg
{
  Int size_;
  
  Char type_;

  Int id_;
  Int extNameSize_;

  Char extName_[19];
  
  Int pathSize_;
  
  Char path_[MAX_PATH];

  
  SftpStatvfsMsg(Int id, const Char *path);
}
__attribute__((packed));


struct SftpStatvfsReplyMsg
{
  Int size_;
  
  Char type_;
  
  Int id_;
  
  Int64 bsize_;
  Int64 frsize_;
  Int64 blocks_;
  Int64 bfree_;
  Int64 bavail_;
  Int64 files_;
  Int64 ffree_;
  Int64 favail_;
  Int64 fsid_;
  Int64 flags_;
  Int64 namemax_;  
}
__attribute__((packed));

//
// NX request.
//
// <request name> <target path>.
//

struct SftpNxMsg
{
  Int size_;
  
  Char type_;

  Int id_;
  Int extNameSize_;

  Char extName_[16];

  Int pathSize_;
  
  Char path_[MAX_PATH];
  
  SftpNxMsg(Int id, const Char *name, const Char *path);
}
__attribute__((packed));


//
// Size of Sftp string message EXCLUDING string data.
//

#define SFTP_STR_HEAD_SIZE ( sizeof(Int)  /* size_   */  \
                           + sizeof(Char) /* type_   */  \
                           + sizeof(Int)  /* id_     */  \
                           + sizeof(Int)) /* strLen_ */

#endif
