/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2016 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXSSHFS, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef Utils_H
#define Utils_H

#undef ERROR
#undef STRING

#include <windows.h>
#include <shlwapi.h>
#include <ntstatus.h>
#include <io.h>
#include <psapi.h>
#include <aclapi.h>
#include <Sddl.h>

#include "Types.h"
#include "Debug.h"

#define NXDEVICED_PIPE "\\\\.\\pipe\\nxdevice-bc72a506-0e0c-4bff-b6f8-c2b6aa32e906"

//
// Definitions.
//

#define LOW_DWORD(X) ((ULONGLONG) (X) & 0xffffffff)

#define HIGH_DWORD(X) ((ULONGLONG) (X) >> 32)

#define MIN(X, Y) ((X) < (Y)) ? (X) : (Y)
#define MAX(X, Y) ((X) > (Y)) ? (X) : (Y)

#define XFREE(X) if (X) free(X)

#define PATH_READY_TIMEOUT 5000
#define OBJECT_FILE_TYPE   0x1c
#define UNLIMITED_COUNT    0x7fffffff

#define SystemHandleInformationClass 16

#define NAMED_PIPE_MASK (1 << 17)

Int UtilsInit();

void UtilsCleanup();

//
// Time functions.
//

double GetTimeMs();

//
// Raw buffer management functions.
//

Int32 InvertBytes(Int32 x);

Int64 InvertBytes64(Int64 x);

DWORD PopInt32(Char *&p);

void PopString(Char *outString, Char *&p, Int *readed, Int capacity);

void SkipString(Char *&p);

//
// Security functions.
//

Int SetObjectRights(const wchar_t *objName, const Char *rights, Int inherit = 1);

//
// Code page conversion functions.
//

Int CovertUtf8ToLocal8(Char *local8, Int local8Size, const Char *utf8);

//
// Path processing functions.
//

Int GetRelativePath(Char *normPath, Int pathSize, Char *relative);

void NormalizePath(wchar_t *path, Int pathSize, wchar_t slash);

Int PathIsReparsePoint(const wchar_t *path);

Int PathIsEmpty(const wchar_t *path);

Int GetParentPath(wchar_t *parent, Int parentSize, const wchar_t *path);

Int ExpandRelativePath(Char *path, Int pathSize, const Char *relative, Int quiet = 0);

const Char *GetLogPath(const Char *postfix);

//
// Functions for detecting third processes, 
// which lock mount point candidate.
// 

Int GetRefferenceCount(const wchar_t *path, Int maxCount = UNLIMITED_COUNT, 
                           Int waitUntilReady = 0);
                                   
Int GetFirstOwner(const wchar_t *path, Char *name, Int nameSize, Int *pid);

Int IsPathBusy(const wchar_t *path);

Int WaitUntilPathReady(const Char *path, Int timeout = PATH_READY_TIMEOUT);

//
// Function to comunicate with device dameon directly via pipe.
//

Int DeviceDaemonSend(Int *daemonResult, Char *response,
                         Int responseSize, const char *fmt, ...);

//
// Workaround for atomic read/write without overlapped mode.
//

struct AtomicReadInfo
{
  HANDLE handle;
  
  Char *buf;
  
  Int bufSize;
};

Int AtomicReadWorker(AtomicReadInfo *ari);

Int AtomicRead(Char *buf, Int bufSize, HANDLE handle, DWORD timeout);

//
// Typedef.
//

typedef LONG NTSTATUS, *PNTSTATUS;

//
// Enums.
//

typedef enum _OBJECT_INFORMATION_CLASS 
{
  ObjectBasicInformation,
  ObjectNameInformation,
  ObjectTypeInformation,
  ObjectAllTypesInformation,
  ObjectHandleInformation
} 
OBJECT_INFORMATION_CLASS;

typedef enum _POOL_TYPE 
{
  NonPagedPool,
  PagedPool,
  NonPagedPoolMustSucceed,
  DontUseThisType,
  NonPagedPoolCacheAligned,
  PagedPoolCacheAligned,
  NonPagedPoolCacheAlignedMustS,
  MaxPoolType,
  NonPagedPoolSession = 32,
  PagedPoolSession,
  NonPagedPoolMustSucceedSession,
  DontUseThisTypeSession,
  NonPagedPoolCacheAlignedSession,
  PagedPoolCacheAlignedSession,
  NonPagedPoolCacheAlignedMustSSession
} 
POOL_TYPE;

//
// System information and control 
//

typedef enum _SYSTEM_INFORMATION_CLASS 
{
  SystemInformationClassMin = 0,
  SystemBasicInformation = 0,
  SystemProcessorInformation = 1,
  SystemPerformanceInformation = 2,
  SystemTimeOfDayInformation = 3,
  SystemPathInformation = 4,
  SystemNotImplemented1 = 4,
  SystemProcessInformation = 5,
  SystemProcessesAndThreadsInformation = 5,
  SystemCallCountInfoInformation = 6,
  SystemCallCounts = 6,
  SystemDeviceInformation = 7,
  SystemConfigurationInformation = 7,
  SystemProcessorPerformanceInformation = 8,
  SystemProcessorTimes = 8,
  SystemFlagsInformation = 9,
  SystemGlobalFlag = 9,
  SystemCallTimeInformation = 10,
  SystemNotImplemented2 = 10,
  SystemModuleInformation = 11,
  SystemLocksInformation = 12,
  SystemLockInformation = 12,
  SystemStackTraceInformation = 13,
  SystemNotImplemented3 = 13,
  SystemPagedPoolInformation = 14,
  SystemNotImplemented4 = 14,
  SystemNonPagedPoolInformation = 15,
  SystemNotImplemented5 = 15,
  SystemHandleInformation = 16,
  SystemObjectInformation = 17,
  SystemPageFileInformation = 18,
  SystemPagefileInformation = 18,
  SystemVdmInstemulInformation = 19,
  SystemInstructionEmulationCounts = 19,
  SystemVdmBopInformation = 20,
  SystemInvalidInfoClass1 = 20,
  SystemFileCacheInformation = 21,
  SystemCacheInformation = 21,
  SystemPoolTagInformation = 22,
  SystemInterruptInformation = 23,
  SystemProcessorStatistics = 23,
  SystemDpcBehaviourInformation = 24,
  SystemDpcInformation = 24,
  SystemFullMemoryInformation = 25,
  SystemNotImplemented6 = 25,
  SystemLoadImage = 26,
  SystemUnloadImage = 27,
  SystemTimeAdjustmentInformation = 28,
  SystemTimeAdjustment = 28,
  SystemSummaryMemoryInformation = 29,
  SystemNotImplemented7 = 29,
  SystemNextEventIdInformation = 30,
  SystemNotImplemented8 = 30,
  SystemEventIdsInformation = 31,
  SystemNotImplemented9 = 31,
  SystemCrashDumpInformation = 32,
  SystemExceptionInformation = 33,
  SystemCrashDumpStateInformation = 34,
  SystemKernelDebuggerInformation = 35,
  SystemContextSwitchInformation = 36,
  SystemRegistryQuotaInformation = 37,
  SystemLoadAndCallImage = 38,
  SystemPrioritySeparation = 39,
  SystemPlugPlayBusInformation = 40,
  SystemNotImplemented10 = 40,
  SystemDockInformation = 41,
  SystemNotImplemented11 = 41,
  // SystemPowerInformation = 42, Conflicts with POWER_INFORMATION_LEVEL 1 
  SystemInvalidInfoClass2 = 42,
  SystemProcessorSpeedInformation = 43,
  SystemInvalidInfoClass3 = 43,
  SystemCurrentTimeZoneInformation = 44,
  SystemTimeZoneInformation = 44,
  SystemLookasideInformation = 45,
  SystemSetTimeSlipEvent = 46,
  SystemCreateSession = 47,
  SystemDeleteSession = 48,
  SystemInvalidInfoClass4 = 49,
  SystemRangeStartInformation = 50,
  SystemVerifierInformation = 51,
  SystemAddVerifier = 52,
  SystemSessionProcessesInformation  = 53,
  SystemInformationClassMax
} 
SYSTEM_INFORMATION_CLASS;

//
// Structs.
//

typedef struct _UNICODE_STRING
{
  USHORT Length;
  USHORT MaximumLength;
  PWSTR  Buffer;
} 
UNICODE_STRING, *PUNICODE_STRING;

typedef struct _OBJECT_TYPE_INFORMATION
{
  UNICODE_STRING Name;
  ULONG TotalNumberOfObjects;
  ULONG TotalNumberOfHandles;
  ULONG TotalPagedPoolUsage;
  ULONG TotalNonPagedPoolUsage;
  ULONG TotalNamePoolUsage;
  ULONG TotalHandleTableUsage;
  ULONG HighWaterNumberOfObjects;
  ULONG HighWaterNumberOfHandles;
  ULONG HighWaterPagedPoolUsage;
  ULONG HighWaterNonPagedPoolUsage;
  ULONG HighWaterNamePoolUsage;
  ULONG HighWaterHandleTableUsage;
  ULONG InvalidAttributes;
  GENERIC_MAPPING GenericMapping;
  ULONG ValidAccess;
  BOOLEAN SecurityRequired;
  BOOLEAN MaintainHandleCount;
  USHORT MaintainTypeList;
  POOL_TYPE PoolType;
  ULONG PagedPoolUsage;
  ULONG NonPagedPoolUsage;
} 
OBJECT_TYPE_INFORMATION, *POBJECT_TYPE_INFORMATION;

typedef struct _SYSTEM_HANDLE
{
  ULONG ProcessId;
  BYTE ObjectTypeNumber;
  BYTE Flags;
  USHORT Handle;
  PVOID Object;
  ACCESS_MASK GrantedAccess;
} 
SYSTEM_HANDLE, *PSYSTEM_HANDLE;

typedef struct _SYSTEM_HANDLE_INFORMATION_
{
  ULONG HandleCount;

  SYSTEM_HANDLE Handles[1];
} 
SYSTEM_HANDLE_INFORMATION_, *PSYSTEM_HANDLE_INFORMATION_;

//
// Functions prototypes.
//

typedef NTSTATUS WINAPI
    (*ZwQuerySystemInformationProto)(SYSTEM_INFORMATION_CLASS,
                                         PVOID, ULONG, PULONG);

//
// Kernel objects related functions.
//

extern "C"
{
NTSTATUS NTAPI 
    NtQueryObject(HANDLE hande, OBJECT_INFORMATION_CLASS infoClass,
                      void *buf, ULONG bufSize, ULONG *written);

NTSTATUS NTAPI 
    NtDuplicateObject(HANDLE SourceProcessHandle, HANDLE SourceHandle,
                          HANDLE TargetProcessHandle, PHANDLE TargetHandle,
                              ACCESS_MASK DesiredAccess,
                                  ULONG HandleAttributes, ULONG Options);
}                      

Int NtQueryObjectEx(HANDLE handle, OBJECT_INFORMATION_CLASS infoClass,
                        void **buf, Int *bufSize);

//
// Enums.
//

enum WindowsVersion
{
  windows_xp,
  windows_vista,
  windows_7,
  windows_unknown
};

//
// Handling process shutdown
//

void StopProcessSafely();

#endif
